@@ -3,7 +3,7 @@
  *                Rujeko Chinomona @ SMU
  * ----------------------------------------------------------------
  * SUNDIALS Copyright Start
- * Copyright (c) 2002-2022, Lawrence Livermore National Security
+ * Copyright (c) 2002-2023, Lawrence Livermore National Security
  * and Southern Methodist University.
  * All rights reserved.
  *
@@ -306,6 +306,7 @@ int main(int argc, char *argv[])
     B->p=2;
     retval = ARKStepSetTables(inner_arkode_mem, 3, 2, NULL, B);
     if (check_retval(&retval, "ARKStepSetTables", 1)) return 1;
+    ARKodeButcherTable_Free(B);
     break;
   case(1):  /* erk-3-3 fast solver (full problem) */
     inner_arkode_mem = ARKStepCreate(fn, NULL, T0, y, ctx);
@@ -325,6 +326,7 @@ int main(int argc, char *argv[])
     B->p=2;
     retval = ARKStepSetTables(inner_arkode_mem, 3, 2, NULL, B);
     if (check_retval(&retval, "ARKStepSetTables", 1)) return 1;
+    ARKodeButcherTable_Free(B);
     break;
   case(9):
   case(5):  /* erk-4-4 fast solver */
@@ -345,6 +347,7 @@ int main(int argc, char *argv[])
     B->q=4;
     retval = ARKStepSetTables(inner_arkode_mem, 4, 0, NULL, B);
     if (check_retval(&retval, "ARKStepSetTables", 1)) return 1;
+    ARKodeButcherTable_Free(B);
     break;
   case(2):  /* esdirk-3-3 fast solver (full problem) */
     inner_arkode_mem = ARKStepCreate(NULL, fn, T0, y, ctx);
@@ -376,6 +379,7 @@ int main(int argc, char *argv[])
     if (check_retval(&retval, "ARKStepSetJacFn", 1)) return 1;
     retval = ARKStepSStolerances(inner_arkode_mem, reltol, abstol);
     if (check_retval(&retval, "ARKStepSStolerances", 1)) return 1;
+    ARKodeButcherTable_Free(B);
     break;
   case(3):  /* no fast dynamics ('evolve' explicitly w/ erk-3-3) */
   case(4):
@@ -396,6 +400,7 @@ int main(int argc, char *argv[])
     B->p=2;
     retval = ARKStepSetTables(inner_arkode_mem, 3, 2, NULL, B);
     if (check_retval(&retval, "ARKStepSetTables", 1)) return 1;
+    ARKodeButcherTable_Free(B);
   }
 
   /* Set the user data pointer */
@@ -459,6 +464,7 @@ int main(int argc, char *argv[])
     if (check_retval((void *)C, "MRIStepCoupling_MIStoMRI", 0)) return 1;
     retval = MRIStepSetCoupling(arkode_mem, C);
     if (check_retval(&retval, "MRIStepSetCoupling", 1)) return 1;
+    ARKodeButcherTable_Free(B);
     break;
   case(4):  /* dirk-2 (trapezoidal), solve-decoupled slow solver */
     arkode_mem = MRIStepCreate(NULL, fn, T0, y, inner_stepper, ctx);
@@ -656,7 +662,6 @@ int main(int argc, char *argv[])
 
   /* Clean up and return */
   N_VDestroy(y);                             /* Free y vector */
-  ARKodeButcherTable_Free(B);                /* Butcher table */
   MRIStepCoupling_Free(C);                   /* free coupling coefficients */
   SUNMatDestroy(Af);                         /* free fast matrix */
   SUNLinSolFree(LSf);                        /* free fast linear solver */
