@@ -3,7 +3,7 @@
  *      Based on cvsRoberts_FSA_dns.c and modified to use KLU
  * -----------------------------------------------------------------
  * SUNDIALS Copyright Start
- * Copyright (c) 2002-2022, Lawrence Livermore National Security
+ * Copyright (c) 2002-2023, Lawrence Livermore National Security
  * and Southern Methodist University.
  * All rights reserved.
  *
@@ -11,8 +11,9 @@
  *
  * SPDX-License-Identifier: BSD-3-Clause
  * SUNDIALS Copyright End
- *-----------------------------------------------------------------
- * Adjoint sensitivity example problem.
+ * -----------------------------------------------------------------
+ * Example problem:
+ *
  * The following is a simple example problem, with the coding
  * needed for its solution by CVODES for Forward Sensitivity
  * Analysis. The problem is from chemical kinetics, and consists
@@ -21,14 +22,13 @@
  *    dy2/dt =  p1*y1 - p2*y2*y3 - p3*(y2)^2
  *    dy3/dt =  p3*(y2)^2
  * on the interval from t = 0.0 to t = 4.e10, with initial
- * conditions y1 = 1.0, y2 = y3 = 0. The reaction rates are: p1=0.04,
- * p2=1e4, and p3=3e7. The problem is stiff.
+ * conditions: y1 = 1.0, y2 = y3 = 0. The problem is stiff.
+ * The reaction rates are: p1=0.04, p2=1e4, and p3=3e7.
  * This program solves the problem with the BDF method, Newton
  * iteration with the KLU linear solver, and a
  * user-supplied Jacobian routine.
  * It uses a scalar relative tolerance and a vector absolute
- * tolerance.
- * Output is printed in decades from t = .4 to t = 4.e10.
+ * tolerance. Output is printed in decades from t = .4 to t = 4.e10.
  * Run statistics (optional outputs) are printed at the end.
  *
  * Optionally, CVODES can compute sensitivities with respect to the
@@ -48,40 +48,47 @@
  *    % cvsRoberts_FSA_klu -sensi sensi_meth err_con
  * where sensi_meth is one of {sim, stg, stg1} and err_con is one of
  * {t, f}.
- * -----------------------------------------------------------------
- */
+ * -----------------------------------------------------------------*/
 
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 
-#include <cvodes/cvodes.h>              /* prototypes for CVODE fcts., consts. */
-#include <nvector/nvector_serial.h>     /* access to serial N_Vector           */
-#include <sunmatrix/sunmatrix_sparse.h> /* access to sparse SUNMatrix          */
-#include <sunlinsol/sunlinsol_klu.h>    /* access to KLU SUNLinearSolver       */
-#include <sundials/sundials_types.h>    /* defs. of realtype, sunindextype     */
-#include <sundials/sundials_math.h>     /* defs. of SUNRabs, SUNRexp, etc.     */
+#include <cvodes/cvodes.h>              /* prototypes for CVODE fcts., consts.  */
+#include <nvector/nvector_serial.h>     /* access to serial N_Vector            */
+#include <sunmatrix/sunmatrix_sparse.h> /* access to sparse SUNMatrix           */
+#include <sunlinsol/sunlinsol_klu.h>    /* access to KLU sparse direct solver   */
+#include <sundials/sundials_types.h>    /* defs. of realtype, sunindextype      */
+#include <sundials/sundials_math.h>     /* defs. of SUNRabs, SUNRexp, etc.      */
 
-/* Accessor macros */
-/* These macros are defined in order to write code with which exactly matched
-   the mathematical problem description given above. */
+/* User-defined vector accessor macro: Ith */
+
+/* These macros are defined in order to write code which exactly matches
+   the mathematical problem description given above.
+
+   Ith(v,i) references the ith component of the vector v, where i is in
+   the range [1..NEQ] and NEQ is defined below. The Ith macro is defined
+   using the N_VIth macro in nvector.h. N_VIth numbers the components of
+   a vector starting from 0. */
+
+#define Ith(v,i)    NV_Ith_S(v,i-1)         /* i-th vector component, i=1..NEQ */
 
-#define Ith(v,i)    NV_Ith_S(v,i-1)       /* i-th vector component, i=1..NEQ */
 
 /* Problem Constants */
 
-#define NEQ   3             /* number of equations  */
-#define Y1    RCONST(1.0)   /* initial y components */
+#define NEQ   3                /* number of equations  */
+#define NNZ   7                /* number of non-zero entries in the Jacobian */
+#define Y1    RCONST(1.0)      /* initial y components */
 #define Y2    RCONST(0.0)
 #define Y3    RCONST(0.0)
-#define RTOL  RCONST(1e-4)  /* scalar relative tolerance */
-#define ATOL1 RCONST(1e-8)  /* vector absolute tolerance components */
-#define ATOL2 RCONST(1e-14)
-#define ATOL3 RCONST(1e-6)
-#define T0    RCONST(0.0)   /* initial time */
-#define T1    RCONST(0.4)   /* first output time */
-#define TMULT RCONST(10.0)  /* output time factor */
-#define NOUT  12            /* number of output times */
+#define RTOL  RCONST(1.0e-4)   /* scalar relative tolerance            */
+#define ATOL1 RCONST(1.0e-8)   /* vector absolute tolerance components */
+#define ATOL2 RCONST(1.0e-14)
+#define ATOL3 RCONST(1.0e-6)
+#define T0    RCONST(0.0)      /* initial time           */
+#define T1    RCONST(0.4)      /* first output time      */
+#define TMULT RCONST(10.0)     /* output time factor     */
+#define NOUT  12               /* number of output times */
 
 #define NP    3             /* number of problem parameters */
 #define NS    3             /* number of sensitivities computed */
@@ -94,7 +101,7 @@ typedef struct {
   realtype p[3];           /* problem parameters */
 } *UserData;
 
-/* Prototypes of user-supplied functions */
+/* Functions Called by the Solver */
 
 static int f(realtype t, N_Vector y, N_Vector ydot, void *user_data);
 
@@ -107,33 +114,42 @@ static int fS(int Ns, realtype t, N_Vector y, N_Vector ydot,
 
 static int ewt(N_Vector y, N_Vector w, void *user_data);
 
+/* Private functions to output results */
+
+static void PrintOutput(void *cvode_mem, realtype t, N_Vector u);
+static void PrintOutputS(N_Vector *uS);
+
+/* Private function to print final statistics */
+
+static void PrintFinalStats(void *cvode_mem, booleantype sensi);
+
 /* Prototypes of private functions */
 
 static void ProcessArgs(int argc, char *argv[],
                         booleantype *sensi, int *sensi_meth,
                         booleantype *err_con);
+
 static void WrongArgs(char *name);
-static void PrintOutput(void *cvode_mem, realtype t, N_Vector u);
-static void PrintOutputS(N_Vector *uS);
-static void PrintFinalStats(void *cvode_mem, booleantype sensi);
+
 static int check_retval(void *returnvalue, const char *funcname, int opt);
 
+
 /*
- *--------------------------------------------------------------------
- * MAIN PROGRAM
- *--------------------------------------------------------------------
+ *-------------------------------
+ * Main Program
+ *-------------------------------
  */
 
 int main(int argc, char *argv[])
 {
   SUNContext sunctx;
+  realtype t, tout;
+  N_Vector y;
   SUNMatrix A;
   SUNLinearSolver LS;
   void *cvode_mem;
+  int retval, iout;
   UserData data;
-  realtype t, tout;
-  N_Vector y;
-  int iout, retval, nnz;
 
   realtype pbar[NS];
   int is;
@@ -141,12 +157,12 @@ int main(int argc, char *argv[])
   booleantype sensi, err_con;
   int sensi_meth;
 
+  data = NULL;
+  y = NULL;
+  yS = NULL;
+  A = NULL;
+  LS = NULL;
   cvode_mem = NULL;
-  data      = NULL;
-  y         = NULL;
-  yS        = NULL;
-  A         = NULL;
-  LS        = NULL;
 
   /* Process arguments */
   ProcessArgs(argc, argv, &sensi, &sensi_meth, &err_con);
@@ -166,23 +182,24 @@ int main(int argc, char *argv[])
   y = N_VNew_Serial(NEQ, sunctx);
   if (check_retval((void *)y, "N_VNew_Serial", 0)) return(1);
 
+  /* Initialize y */
   Ith(y,1) = Y1;
   Ith(y,2) = Y2;
   Ith(y,3) = Y3;
 
   /* Call CVodeCreate to create the solver memory and specify the
-     Backward Differentiation Formula */
+   * Backward Differentiation Formula */
   cvode_mem = CVodeCreate(CV_BDF, sunctx);
   if (check_retval((void *)cvode_mem, "CVodeCreate", 0)) return(1);
 
   /* Call CVodeInit to initialize the integrator memory and specify the
-     user's right hand side function in y'=f(t,y), the initial time T0, and
-     the initial dependent variable vector y. */
+   * user's right hand side function in y'=f(t,y), the initial time T0, and
+   * the initial dependent variable vector y. */
   retval = CVodeInit(cvode_mem, f, T0, y);
   if (check_retval(&retval, "CVodeInit", 1)) return(1);
 
   /* Call CVodeWFtolerances to specify a user-supplied function ewt that sets
-     the multiplicative error weights w_i for use in the weighted RMS norm */
+   * the multiplicative error weights w_i for use in the weighted RMS norm */
   retval = CVodeWFtolerances(cvode_mem, ewt);
   if (check_retval(&retval, "CVodeWFtolerances", 1)) return(1);
 
@@ -191,8 +208,7 @@ int main(int argc, char *argv[])
   if (check_retval(&retval, "CVodeSetUserData", 1)) return(1);
 
   /* Create sparse SUNMatrix for use in linear solves */
-  nnz = NEQ * NEQ; /* max no. of nonzeros entries in the Jac */
-  A = SUNSparseMatrix(NEQ, NEQ, nnz, CSC_MAT, sunctx);
+  A = SUNSparseMatrix(NEQ, NEQ, NNZ, CSC_MAT, sunctx);
   if (check_retval((void *)A, "SUNSparseMatrix", 0)) return(1);
 
   /* Create KLU SUNLinearSolver object */
@@ -203,11 +219,11 @@ int main(int argc, char *argv[])
   retval = CVodeSetLinearSolver(cvode_mem, LS, A);
   if (check_retval(&retval, "CVodeSetLinearSolver", 1)) return(1);
 
-  /* Set the Jacobian routine to Jac (user-supplied) */
+  /* Set the user-supplied Jacobian routine Jac */
   retval = CVodeSetJacFn(cvode_mem, Jac);
   if (check_retval(&retval, "CVodeSetJacFn", 1)) return(1);
 
-  printf("\n3-species chemical kinetics problem\n");
+  printf(" \n3-species kinetics problem\n");
 
   /* Sensitivity-related settings */
   if (sensi) {
@@ -223,26 +239,26 @@ int main(int argc, char *argv[])
     for (is=0;is<NS;is++) N_VConst(ZERO, yS[is]);
 
     /* Call CVodeSensInit1 to activate forward sensitivity computations
-       and allocate internal memory for COVEDS related to sensitivity
-       calculations. Computes the right-hand sides of the sensitivity
-       ODE, one at a time */
+     * and allocate internal memory for COVEDS related to sensitivity
+     * calculations. Computes the right-hand sides of the sensitivity
+     * ODE, one at a time */
     retval = CVodeSensInit1(cvode_mem, NS, sensi_meth, fS, yS);
     if(check_retval(&retval, "CVodeSensInit", 1)) return(1);
 
     /* Call CVodeSensEEtolerances to estimate tolerances for sensitivity
-       variables based on the rolerances supplied for states variables and
-       the scaling factor pbar */
+     * variables based on the rolerances supplied for states variables and
+     * the scaling factor pbar */
     retval = CVodeSensEEtolerances(cvode_mem);
     if(check_retval(&retval, "CVodeSensEEtolerances", 1)) return(1);
 
     /* Set sensitivity analysis optional inputs */
     /* Call CVodeSetSensErrCon to specify the error control strategy for
-       sensitivity variables */
+     * sensitivity variables */
     retval = CVodeSetSensErrCon(cvode_mem, err_con);
     if (check_retval(&retval, "CVodeSetSensErrCon", 1)) return(1);
 
     /* Call CVodeSetSensParams to specify problem parameter information for
-       sensitivity calculations */
+     * sensitivity calculations */
     retval = CVodeSetSensParams(cvode_mem, NULL, pbar, NULL);
     if (check_retval(&retval, "CVodeSetSensParams", 1)) return(1);
 
@@ -261,7 +277,8 @@ int main(int argc, char *argv[])
 
   }
 
-  /* In loop over output points, call CVode, print results, test for error */
+  /* In loop, call CVode, print results, and test for error.
+     Break out of loop when NOUT preset output times have been reached.  */
 
   printf("\n\n");
   printf("===========================================");
@@ -279,7 +296,7 @@ int main(int argc, char *argv[])
     PrintOutput(cvode_mem, t, y);
 
     /* Call CVodeGetSens to get the sensitivity solution vector after a
-       successful return from CVode */
+     * successful return from CVode */
     if (sensi) {
       retval = CVodeGetSens(cvode_mem, &t, yS);
       if (check_retval(&retval, "CVodeGetSens", 1)) break;
@@ -290,32 +307,30 @@ int main(int argc, char *argv[])
 
   }
 
-  /* Print final statistics */
+  /* Print some final statistics */
   PrintFinalStats(cvode_mem, sensi);
 
   /* Free memory */
-
-  N_VDestroy(y);                    /* Free y vector */
-  if (sensi) {
-    N_VDestroyVectorArray(yS, NS);  /* Free yS vector */
-  }
-  free(data);                              /* Free user data */
-  CVodeFree(&cvode_mem);                   /* Free CVODES memory */
-  SUNLinSolFree(LS);                       /* Free the linear solver memory */
-  SUNMatDestroy(A);                        /* Free the matrix memory */
-  SUNContext_Free(&sunctx);                /* Free the SUNDIALS context */
+  N_VDestroy(y);                            /* Free y vector */
+  if (sensi) N_VDestroyVectorArray(yS, NS); /* Free yS vector */
+  free(data);                               /* Free user data */
+  CVodeFree(&cvode_mem);                    /* Free CVODES memory */
+  SUNLinSolFree(LS);                        /* Free the linear solver memory */
+  SUNMatDestroy(A);                         /* Free the matrix memory */
+  SUNContext_Free(&sunctx);                 /* Free the SUNDIALS context */
 
   return(0);
 }
 
+
 /*
- *--------------------------------------------------------------------
- * FUNCTIONS CALLED BY CVODES
- *--------------------------------------------------------------------
+ *-------------------------------
+ * Functions called by the solver
+ *-------------------------------
  */
 
 /*
- * f routine. Compute f(t,y).
+ * f routine. Compute function f(t,y).
  */
 
 static int f(realtype t, N_Vector y, N_Vector ydot, void *user_data)
@@ -336,58 +351,66 @@ static int f(realtype t, N_Vector y, N_Vector ydot, void *user_data)
 }
 
 /*
- * Jacobian routine. Compute J(t,y).
+ * Jacobian routine. Compute J(t,y) = df/dy. *
  */
 
 static int Jac(realtype t, N_Vector y, N_Vector fy, SUNMatrix J,
                void *user_data, N_Vector tmp1, N_Vector tmp2, N_Vector tmp3)
 {
-  realtype *yval;
-  sunindextype *colptrs = SUNSparseMatrix_IndexPointers(J);
+  /* State at which to evaluate the Jacobian */
+  realtype *yval = N_VGetArrayPointer(y);
+
+  /* J is stored in CSC format:
+     data    = non-zero matrix entries stored column-wise (length NNZ)
+     rowvals = row index for each non-zero matrix entry (length NNZ)
+     colptrs = i-th entry is the index in data where the first non-zero matrix
+               entry of the i-th column is stored (length NEQ + 1) */
+  realtype     *data    = SUNSparseMatrix_Data(J);
   sunindextype *rowvals = SUNSparseMatrix_IndexValues(J);
-  realtype *data = SUNSparseMatrix_Data(J);
-  UserData userdata;
-  realtype p1, p2, p3;
-
-  yval = N_VGetArrayPointer(y);
-
-  userdata = (UserData) user_data;
-  p1 = userdata->p[0]; p2 = userdata->p[1]; p3 = userdata->p[2];
-
-  SUNMatZero(J);
+  sunindextype *colptrs = SUNSparseMatrix_IndexPointers(J);
+  UserData     userdata = (UserData) user_data;
+  realtype     p1 = userdata->p[0];
+  realtype     p2 = userdata->p[1];
+  realtype     p3 = userdata->p[2];
 
+  /* first column entries start at data[0], two entries (rows 0 and 1) */
   colptrs[0] = 0;
-  colptrs[1] = 3;
-  colptrs[2] = 6;
-  colptrs[3] = 9;
 
-  data[0] = -p1;
   rowvals[0] = 0;
-  data[1] = p1;
+  data[0]    = -p1;
+
   rowvals[1] = 1;
-  data[2] = ZERO;
-  rowvals[2] = 2;
+  data[1]    = p1;
 
-  data[3] = p2*yval[2];
-  rowvals[3] = 0;
-  data[4] = -p2*yval[2]-2*p3*yval[1];
-  rowvals[4] = 1;
-  data[5] = 2*yval[1];
-  rowvals[5] = 2;
+  /* second column entries start at data[2], three entries (rows 0, 1, and 2) */
+  colptrs[1] = 2;
 
-  data[6] = p2*yval[1];
-  rowvals[6] = 0;
-  data[7] = -p2*yval[1];
-  rowvals[7] = 1;
-  data[8] = ZERO;
-  rowvals[8] = 2;
+  rowvals[2] = 0;
+  data[2]    = p2 * yval[2];
+
+  rowvals[3] = 1;
+  data[3]    = (-p2 * yval[2]) - (2 * p3 * yval[1]);
+
+  rowvals[4] = 2;
+  data[4]    = 2 * p3 * yval[1];
+
+  /* third column entries start at data[5], two entries (rows 0 and 1) */
+  colptrs[2] = 5;
+
+  rowvals[5] = 0;
+  data[5]    = p2 * yval[1];
+
+  rowvals[6] = 1;
+  data[6]    = -p2 * yval[1];
+
+  /* number of non-zeros */
+  colptrs[3] = 7;
 
   return(0);
 }
 
 /*
- * fS routine. Compute sensitivity r.h.s. CVSensRhs1Fn is compatible with any
- * valid value of the argument ism to CVodeSensInit and CVodeSensInit1
+ * fS routine. Compute sensitivity r.h.s.
  */
 
 static int fS(int Ns, realtype t, N_Vector y, N_Vector ydot,
@@ -457,9 +480,9 @@ static int ewt(N_Vector y, N_Vector w, void *user_data)
 }
 
 /*
- *--------------------------------------------------------------------
- * PRIVATE FUNCTIONS
- *--------------------------------------------------------------------
+ *-------------------------------
+ * Private helper functions
+ *-------------------------------
  */
 
 /*
@@ -597,15 +620,13 @@ static void PrintOutputS(N_Vector *uS)
 }
 
 /*
- * Print some final statistics from the CVODES memory.
+ * Get and print some final statistics
  */
 
 static void PrintFinalStats(void *cvode_mem, booleantype sensi)
 {
-  long int nst;
-  long int nfe, nsetups, nni, ncfn, netf;
-  long int nfSe, nfeS, nsetupsS, nniS, ncfnS, netfS;
-  long int nje;
+  long int nst, nfe, nsetups, nje, nni, nnf, ncfn, netf;
+  long int nfSe, nfeS, nsetupsS, nniS, nnfS, ncfnS, netfS;
   int retval;
 
   retval = CVodeGetNumSteps(cvode_mem, &nst);
@@ -618,8 +639,10 @@ static void PrintFinalStats(void *cvode_mem, booleantype sensi)
   check_retval(&retval, "CVodeGetNumErrTestFails", 1);
   retval = CVodeGetNumNonlinSolvIters(cvode_mem, &nni);
   check_retval(&retval, "CVodeGetNumNonlinSolvIters", 1);
-  retval = CVodeGetNumNonlinSolvConvFails(cvode_mem, &ncfn);
+  retval = CVodeGetNumNonlinSolvConvFails(cvode_mem, &nnf);
   check_retval(&retval, "CVodeGetNumNonlinSolvConvFails", 1);
+  retval = CVodeGetNumStepSolveFails(cvode_mem, &ncfn);
+  check_retval(&retval, "CVodeGetNumStepSolveFails", 1);
 
   if (sensi) {
     retval = CVodeGetSensNumRhsEvals(cvode_mem, &nfSe);
@@ -632,39 +655,37 @@ static void PrintFinalStats(void *cvode_mem, booleantype sensi)
     check_retval(&retval, "CVodeGetSensNumErrTestFails", 1);
     retval = CVodeGetSensNumNonlinSolvIters(cvode_mem, &nniS);
     check_retval(&retval, "CVodeGetSensNumNonlinSolvIters", 1);
-    retval = CVodeGetSensNumNonlinSolvConvFails(cvode_mem, &ncfnS);
+    retval = CVodeGetSensNumNonlinSolvConvFails(cvode_mem, &nnfS);
     check_retval(&retval, "CVodeGetSensNumNonlinSolvConvFails", 1);
+    retval = CVodeGetNumStepSensSolveFails(cvode_mem, &ncfnS);
+    check_retval(&retval, "CVodeGetNumStepSensSolveFails", 1);
   }
 
   retval = CVodeGetNumJacEvals(cvode_mem, &nje);
   check_retval(&retval, "CVodeGetNumJacEvals", 1);
 
-  printf("\nFinal Statistics\n\n");
-  printf("nst     = %5ld\n\n", nst);
-  printf("nfe     = %5ld\n",   nfe);
-  printf("netf    = %5ld    nsetups  = %5ld\n", netf, nsetups);
-  printf("nni     = %5ld    ncfn     = %5ld\n", nni, ncfn);
+  printf("\nFinal Statistics:\n");
+  printf("nst = %-6ld nfe = %-6ld nsetups = %-6ld nje = %ld\n",
+         nst, nfe, nsetups, nje);
+  printf("nni = %-6ld nnf = %-6ld netf = %-6ld    ncfn = %-6ld\n\n",
+         nni, nnf, netf, ncfn);
 
   if(sensi) {
-    printf("\n");
-    printf("nfSe    = %5ld    nfeS     = %5ld\n", nfSe, nfeS);
-    printf("netfs   = %5ld    nsetupsS = %5ld\n", netfS, nsetupsS);
-    printf("nniS    = %5ld    ncfnS    = %5ld\n", nniS, ncfnS);
+    printf("nfSe = %-6ld nfeS = %-6ld nsetupsS = %-6ld\n",
+           nfSe, nfeS, nsetupsS);
+    printf("nniS = %-6ld nnfS = %-6ld netfS = %-6ld ncfnS = %-6ld\n\n",
+           nniS, nnfS, netfS, ncfnS);
   }
-
-  printf("\n");
-  printf("nje    = %5ld\n", nje);
-
 }
 
 /*
- * Check function return value.
- *    opt == 0 means SUNDIALS function allocates memory so check if
- *             returned NULL pointer
- *    opt == 1 means SUNDIALS function returns an integer value so check if
- *             retval < 0
- *    opt == 2 means function allocates memory so check if returned
- *             NULL pointer
+ * Check function return value...
+ *   opt == 0 means SUNDIALS function allocates memory so check if
+ *            returned NULL pointer
+ *   opt == 1 means SUNDIALS function returns an integer value so check if
+ *            retval < 0
+ *   opt == 2 means function allocates memory so check if returned
+ *            NULL pointer
  */
 
 static int check_retval(void *returnvalue, const char *funcname, int opt)
@@ -673,25 +694,22 @@ static int check_retval(void *returnvalue, const char *funcname, int opt)
 
   /* Check if SUNDIALS function returned NULL pointer - no memory allocated */
   if (opt == 0 && returnvalue == NULL) {
-    fprintf(stderr,
-            "\nSUNDIALS_ERROR: %s() failed - returned NULL pointer\n\n",
-	    funcname);
+    fprintf(stderr, "\nSUNDIALS_ERROR: %s() failed - returned NULL pointer\n\n",
+            funcname);
     return(1); }
 
   /* Check if retval < 0 */
   else if (opt == 1) {
     retval = (int *) returnvalue;
     if (*retval < 0) {
-      fprintf(stderr,
-              "\nSUNDIALS_ERROR: %s() failed with retval = %d\n\n",
-	      funcname, *retval);
+      fprintf(stderr, "\nSUNDIALS_ERROR: %s() failed with retval = %d\n\n",
+              funcname, *retval);
       return(1); }}
 
   /* Check if function returned NULL pointer - no memory allocated */
   else if (opt == 2 && returnvalue == NULL) {
-    fprintf(stderr,
-            "\nMEMORY_ERROR: %s() failed - returned NULL pointer\n\n",
-	    funcname);
+    fprintf(stderr, "\nMEMORY_ERROR: %s() failed - returned NULL pointer\n\n",
+            funcname);
     return(1); }
 
   return(0);
