@@ -4,7 +4,7 @@
  *      Based on idaFoodWeb_bnd.c and parallelized with OpenMP
  * -----------------------------------------------------------------
  * SUNDIALS Copyright Start
- * Copyright (c) 2002-2019, Lawrence Livermore National Security
+ * Copyright (c) 2002-2023, Lawrence Livermore National Security
  * and Southern Methodist University.
  * All rights reserved.
  *
@@ -206,6 +206,7 @@ int main(int argc, char *argv[])
   sunindextype mu, ml;
   realtype rtol, atol, t0, tout, tret;
   int num_threads;
+  SUNContext ctx;
 
   ida_mem = NULL;
   A = NULL;
@@ -221,24 +222,28 @@ int main(int argc, char *argv[])
   if (argc > 1)      /* overwrite with command line value, if supplied */
     num_threads = (int) strtol(argv[1], NULL, 0);
 
+  /* Create the SUNDIALS context object for this simulation */
+  retval = SUNContext_Create(NULL, &ctx);
+  if (check_retval(&retval, "SUNContext_Create", 1)) return 1;
+
   /* Allocate and initialize user data block webdata. */
 
   webdata = (UserData) malloc(sizeof *webdata);
-  webdata->rates = N_VNew_OpenMP(NEQ, num_threads);
-  webdata->acoef = newDenseMat(NUM_SPECIES, NUM_SPECIES);
+  webdata->rates = N_VNew_OpenMP(NEQ, num_threads, ctx);
+  webdata->acoef = SUNDlsMat_newDenseMat(NUM_SPECIES, NUM_SPECIES);
   webdata->nthreads = num_threads;
 
   InitUserData(webdata);
 
   /* Allocate N-vectors and initialize cc, cp, and id. */
 
-  cc  = N_VNew_OpenMP(NEQ, num_threads);
+  cc  = N_VNew_OpenMP(NEQ, num_threads, ctx);
   if(check_retval((void *)cc, "N_VNew_OpenMP", 0)) return(1);
 
-  cp  = N_VNew_OpenMP(NEQ, num_threads);
+  cp  = N_VNew_OpenMP(NEQ, num_threads, ctx);
   if(check_retval((void *)cp, "N_VNew_OpenMP", 0)) return(1);
 
-  id  = N_VNew_OpenMP(NEQ, num_threads);
+  id  = N_VNew_OpenMP(NEQ, num_threads, ctx);
   if(check_retval((void *)id, "N_VNew_OpenMP", 0)) return(1);
 
   SetInitialProfiles(cc, cp, id, webdata);
@@ -251,7 +256,7 @@ int main(int argc, char *argv[])
 
   /* Call IDACreate and IDAMalloc to initialize IDA. */
 
-  ida_mem = IDACreate();
+  ida_mem = IDACreate(ctx);
   if(check_retval((void *) ida_mem, "IDACreate", 0)) return(1);
 
   retval = IDASetUserData(ida_mem, webdata);
@@ -269,9 +274,9 @@ int main(int argc, char *argv[])
   /* Setup band matrix and linear solver, and attach to IDA. */
 
   mu = ml = NSMX;
-  A = SUNBandMatrix(NEQ, mu, ml);
+  A = SUNBandMatrix(NEQ, mu, ml, ctx);
   if(check_retval((void *)A, "SUNBandMatrix", 0)) return(1);
-  LS = SUNLinSol_Band(cc, A);
+  LS = SUNLinSol_Band(cc, A, ctx);
   if(check_retval((void *)LS, "SUNLinSol_Band", 0)) return(1);
   retval = IDASetLinearSolver(ida_mem, LS, A);
   if(check_retval(&retval, "IDASetLinearSolver", 1)) return(1);
@@ -316,10 +321,12 @@ int main(int argc, char *argv[])
   N_VDestroy_OpenMP(id);
 
 
-  destroyMat(webdata->acoef);
+  SUNDlsMat_destroyMat(webdata->acoef);
   N_VDestroy_OpenMP(webdata->rates);
   free(webdata);
 
+  SUNContext_Free(&ctx);
+
   return(0);
 }
 
@@ -350,11 +357,13 @@ static int resweb(realtype tt, N_Vector cc, N_Vector cp,
   realtype *resv, *cpv;
   UserData webdata;
 
+  jx = jy = is = 0;
+
   webdata = (UserData)user_data;
 
-  cpv = NV_DATA_OMP(cp);
+  cpv  = NV_DATA_OMP(cp);
   resv = NV_DATA_OMP(res);
-  np = webdata->np;
+  np   = webdata->np;
 
   /* Call Fweb to set res to vector of right-hand sides. */
   Fweb(tt, cc, res, webdata);
@@ -609,6 +618,8 @@ static void Fweb(realtype tcalc, N_Vector cc, N_Vector crate,
   /* Loop over grid points, evaluate interaction vector (length ns),
      form diffusion difference terms, and load crate.                    */
 
+  jx = jy = is = 0;
+
   for (jy = 0; jy < MY; jy++) {
     yy = (webdata->dy) * jy ;
     idyu = (jy!=MY-1) ? NSMX : -NSMX;
