@@ -3,7 +3,7 @@
  * Programmer(s): Daniel R. Reynolds and Ting Yan @ SMU
  * -----------------------------------------------------------------
  * SUNDIALS Copyright Start
- * Copyright (c) 2002-2019, Lawrence Livermore National Security
+ * Copyright (c) 2002-2023, Lawrence Livermore National Security
  * and Southern Methodist University.
  * All rights reserved.
  *
@@ -221,6 +221,7 @@ int main(int argc, char *argv[])
   int maxl;
   realtype rtol, atol, t0, tout, tret;
   int num_threads;
+  SUNContext ctx;
 
   ida_mem = NULL;
   LS = NULL;
@@ -235,16 +236,20 @@ int main(int argc, char *argv[])
   if (argc > 1)      /* overwrithe with command line value, if supplied */
     num_threads = (int) strtol(argv[1], NULL, 0);
 
+  /* Create the SUNDIALS context object for this simulation */
+  retval = SUNContext_Create(NULL, &ctx);
+  if (check_retval(&retval, "SUNContext_Create", 1)) return 1;
+
   /* Allocate and initialize user data block webdata. */
 
   webdata = (UserData) malloc(sizeof *webdata);
-  webdata->rates = N_VNew_OpenMP(NEQ, num_threads);
-  webdata->acoef = newDenseMat(NUM_SPECIES, NUM_SPECIES);
-  webdata->ewt = N_VNew_OpenMP(NEQ, num_threads);
+  webdata->rates = N_VNew_OpenMP(NEQ, num_threads, ctx);
+  webdata->acoef = SUNDlsMat_newDenseMat(NUM_SPECIES, NUM_SPECIES);
+  webdata->ewt = N_VNew_OpenMP(NEQ, num_threads, ctx);
   for (jx = 0; jx < MX; jx++) {
     for (jy = 0; jy < MY; jy++) {
-      (webdata->pivot)[jx][jy] = newIndexArray(NUM_SPECIES);
-      (webdata->PP)[jx][jy] = newDenseMat(NUM_SPECIES, NUM_SPECIES);
+      (webdata->pivot)[jx][jy] = SUNDlsMat_newIndexArray(NUM_SPECIES);
+      (webdata->PP)[jx][jy] = SUNDlsMat_newDenseMat(NUM_SPECIES, NUM_SPECIES);
     }
   }
   webdata->nthreads = num_threads;
@@ -253,13 +258,13 @@ int main(int argc, char *argv[])
 
   /* Allocate N-vectors and initialize cc, cp, and id. */
 
-  cc  = N_VNew_OpenMP(NEQ, num_threads);
+  cc  = N_VNew_OpenMP(NEQ, num_threads, ctx);
   if(check_retval((void *)cc, "N_VNew_OpenMP", 0)) return(1);
 
-  cp  = N_VNew_OpenMP(NEQ, num_threads);
+  cp  = N_VNew_OpenMP(NEQ, num_threads, ctx);
   if(check_retval((void *)cp, "N_VNew_OpenMP", 0)) return(1);
 
-  id  = N_VNew_OpenMP(NEQ, num_threads);
+  id  = N_VNew_OpenMP(NEQ, num_threads, ctx);
   if(check_retval((void *)id, "N_VNew_OpenMP", 0)) return(1);
 
   SetInitialProfiles(cc, cp, id, webdata);
@@ -272,7 +277,7 @@ int main(int argc, char *argv[])
 
   /* Call IDACreate and IDAMalloc to initialize IDA. */
 
-  ida_mem = IDACreate();
+  ida_mem = IDACreate(ctx);
   if(check_retval((void *)ida_mem, "IDACreate", 0)) return(1);
 
   retval = IDASetUserData(ida_mem, webdata);
@@ -293,7 +298,7 @@ int main(int argc, char *argv[])
      preconditioning routines. */
 
   maxl = 16;                               /* max dimension of the Krylov subspace */
-  LS = SUNLinSol_SPGMR(cc, PREC_LEFT, maxl);      /* IDA only allows left preconditioning */
+  LS = SUNLinSol_SPGMR(cc, SUN_PREC_LEFT, maxl, ctx);      /* IDA only allows left preconditioning */
   if(check_retval((void *)LS, "SUNLinSol_SPGMR", 0)) return(1);
 
   retval = IDASetLinearSolver(ida_mem, LS, NULL);
@@ -341,17 +346,19 @@ int main(int argc, char *argv[])
   N_VDestroy(id);
 
 
-  destroyMat(webdata->acoef);
+  SUNDlsMat_destroyMat(webdata->acoef);
   N_VDestroy(webdata->rates);
   N_VDestroy(webdata->ewt);
   for (jx = 0; jx < MX; jx++) {
     for (jy = 0; jy < MY; jy ++) {
-      destroyArray((webdata->pivot)[jx][jy]);
-      destroyMat((webdata->PP)[jx][jy]);
+      SUNDlsMat_destroyArray((webdata->pivot)[jx][jy]);
+      SUNDlsMat_destroyMat((webdata->PP)[jx][jy]);
     }
   }
   free(webdata);
 
+  SUNContext_Free(&ctx);
+
   return(0);
 }
 
@@ -382,11 +389,13 @@ static int resweb(realtype tt, N_Vector cc, N_Vector cp,
   realtype *resv, *cpv;
   UserData webdata;
 
+  jx = jy = is = 0;
+
   webdata = (UserData)user_data;
 
-  cpv = NV_DATA_OMP(cp);
+  cpv  = NV_DATA_OMP(cp);
   resv = NV_DATA_OMP(res);
-  np = webdata->np;
+  np   = webdata->np;
 
   /* Call Fweb to set res to vector of right-hand sides. */
   Fweb(tt, cc, res, webdata);
@@ -469,7 +478,7 @@ static int Precond(realtype tt, N_Vector cc, N_Vector cp,
 	cxy[js] = cctmp;
       }
 
-      ret = denseGETRF(Pxy, NUM_SPECIES, NUM_SPECIES, (webdata->pivot)[jx][jy]);
+      ret = SUNDlsMat_denseGETRF(Pxy, NUM_SPECIES, NUM_SPECIES, (webdata->pivot)[jx][jy]);
 
       if (ret != 0) return(1);
     }
@@ -486,9 +495,11 @@ static int PSolve(realtype tt, N_Vector cc, N_Vector cp,
 {
   realtype **Pxy, *zxy;
   sunindextype *pivot;
-  int jx, jy;
+  sunindextype jx, jy;
   UserData webdata;
 
+  jx = jy = 0;
+
   webdata = (UserData) user_data;
 
   N_VScale(ONE, rvec, zvec);
@@ -500,7 +511,7 @@ static int PSolve(realtype tt, N_Vector cc, N_Vector cp,
       zxy = IJ_Vptr(zvec, jx, jy);
       Pxy = (webdata->PP)[jx][jy];
       pivot = (webdata->pivot)[jx][jy];
-      denseGETRS(Pxy, NUM_SPECIES, pivot, zxy);
+      SUNDlsMat_denseGETRS(Pxy, NUM_SPECIES, pivot, zxy);
     }
   }
 
@@ -738,6 +749,8 @@ static void Fweb(realtype tcalc, N_Vector cc, N_Vector crate,
   /* Loop over grid points, evaluate interaction vector (length ns),
      form diffusion difference terms, and load crate.                    */
 
+  jx = jy = is = 0;
+
   for (jy = 0; jy < MY; jy++) {
     yy = (webdata->dy) * jy ;
     idyu = (jy!=MY-1) ? NSMX : -NSMX;
