@@ -4,7 +4,7 @@
  *         Allan Taylor, Alan Hindmarsh and Radu Serban @ LLNL
  * -----------------------------------------------------------------
  * SUNDIALS Copyright Start
- * Copyright (c) 2002-2021, Lawrence Livermore National Security
+ * Copyright (c) 2002-2023, Lawrence Livermore National Security
  * and Southern Methodist University.
  * All rights reserved.
  *
@@ -130,6 +130,7 @@ static int check_retval(void *returnvalue, const char *funcname, int opt, int id
 int main(int argc, char *argv[])
 {
   MPI_Comm comm;
+  SUNContext ctx;
   void *ida_mem;
   SUNLinearSolver LS;
   UserData data;
@@ -159,6 +160,11 @@ int main(int argc, char *argv[])
     return(1);
   }
 
+  /* Create the SUNDIALS context object for this simulation. */
+
+  retval = SUNContext_Create((void*) &comm, &ctx);
+  if (check_retval(&retval, "SUNContext_Create", 1, thispe)) MPI_Abort(comm, 1);
+
   /* Set local length local_N and global length Neq. */
 
   local_N = MXSUB*MYSUB;
@@ -171,28 +177,28 @@ int main(int argc, char *argv[])
   if(check_retval((void *)data, "malloc", 2, thispe))
     MPI_Abort(comm, 1);
 
-  uu = N_VNew_Parallel(comm, local_N, Neq);
+  uu = N_VNew_Parallel(comm, local_N, Neq, ctx);
   if(check_retval((void *)uu, "N_VNew_Parallel", 0, thispe))
     MPI_Abort(comm, 1);
 
-  up = N_VNew_Parallel(comm, local_N, Neq);
+  up = N_VClone(uu);
   if(check_retval((void *)up, "N_VNew_Parallel", 0, thispe))
     MPI_Abort(comm, 1);
 
-  res = N_VNew_Parallel(comm, local_N, Neq);
+  res = N_VClone(uu);
   if(check_retval((void *)res, "N_VNew_Parallel", 0, thispe))
     MPI_Abort(comm, 1);
 
-  constraints = N_VNew_Parallel(comm, local_N, Neq);
+  constraints = N_VClone(uu);
   if(check_retval((void *)constraints, "N_VNew_Parallel", 0, thispe))
     MPI_Abort(comm, 1);
 
-  id = N_VNew_Parallel(comm, local_N, Neq);
+  id = N_VClone(uu);
   if(check_retval((void *)id, "N_VNew_Parallel", 0, thispe))
     MPI_Abort(comm, 1);
 
   /* An N-vector to hold preconditioner. */
-  data->pp = N_VNew_Parallel(comm, local_N, Neq);
+  data->pp = N_VClone(uu);
   if(check_retval((void *)data->pp, "N_VNew_Parallel", 0, thispe))
     MPI_Abort(comm, 1);
 
@@ -215,7 +221,7 @@ int main(int argc, char *argv[])
 
   /* Call IDACreate and IDAMalloc to initialize solution. */
 
-  ida_mem = IDACreate();
+  ida_mem = IDACreate(ctx);
   if(check_retval((void *)ida_mem, "IDACreate", 0, thispe)) MPI_Abort(comm, 1);
 
   retval = IDASetUserData(ida_mem, data);
@@ -239,7 +245,7 @@ int main(int argc, char *argv[])
 
   /* Call SUNLinSol_SPGMR and IDASetLinearSolver to specify the linear solver. */
 
-  LS = SUNLinSol_SPGMR(uu, PREC_LEFT, 0);  /* use default maxl */
+  LS = SUNLinSol_SPGMR(uu, SUN_PREC_LEFT, 0, ctx);  /* use default maxl */
   if(check_retval((void *)LS, "SUNLinSol_SPGMR", 0, thispe)) MPI_Abort(comm, 1);
 
   retval = IDASetLinearSolver(ida_mem, LS, NULL);
@@ -281,6 +287,8 @@ int main(int argc, char *argv[])
   N_VDestroy(data->pp);
   free(data);
 
+  SUNContext_Free(&ctx);
+
   MPI_Finalize();
 
   return(0);
