@@ -3,7 +3,7 @@
  *      Based on kinFoodWeb_kry.c and parallelized with OpenMP
  * -----------------------------------------------------------------
  * SUNDIALS Copyright Start
- * Copyright (c) 2002-2019, Lawrence Livermore National Security
+ * Copyright (c) 2002-2023, Lawrence Livermore National Security
  * and Southern Methodist University.
  * All rights reserved.
  *
@@ -76,7 +76,7 @@
  *
  * If the user wants to use the default value or the number of
  * threads set by the environment variable use
- *      % ./kinFoodWeb_kry_omp 
+ *      % ./kinFoodWeb_kry_omp
  * If the user wants to specify the number of threads to use
  *      % ./kinFoodWeb_kry_omp num_threads
  * where num_threads is the number of threads the user wants to use
@@ -107,18 +107,23 @@
 #include <sunlinsol/sunlinsol_spgmr.h> /* access to SPGMR SUNLinearSolver      */
 #include <sundials/sundials_dense.h>   /* use generic dense solver in precond. */
 #include <sundials/sundials_types.h>   /* defs. of realtype, sunindextype      */
-#include <sundials/sundials_math.h>    /* access to SUNMAX, SUNRabs, SUNRsqrt  */
 
 #ifdef _OPENMP
 #include <omp.h>
 #endif
 
+/* helpful macros */
+
+#ifndef MAX
+#define MAX(A, B) ((A) > (B) ? (A) : (B))
+#endif
+
 /* Problem Constants */
 
 #define NUM_SPECIES     6  /* must equal 2*(number of prey or predators)
-                              number of prey = number of predators       */ 
+                              number of prey = number of predators       */
 
-#define PI       RCONST(3.1415926535898)   /* pi */ 
+#define PI       RCONST(3.1415926535898)   /* pi */
 
 #define MX          8              /* MX = number of x mesh points */
 #define MY          8              /* MY = number of y mesh points */
@@ -146,12 +151,12 @@
 
 /* IJ_Vptr is defined in order to translate from the underlying 3D structure
    of the dependent variable vector to the 1D storage scheme for an N-vector.
-   IJ_Vptr(vv,i,j) returns a pointer to the location in vv corresponding to 
+   IJ_Vptr(vv,i,j) returns a pointer to the location in vv corresponding to
    indices is = 0, jx = i, jy = j.    */
 
 #define IJ_Vptr(vv,i,j)   (&NV_Ith_OMP(vv, i*NUM_SPECIES + j*NSMX))
 
-/* Type : UserData 
+/* Type : UserData
    contains preconditioner blocks, pivot arrays, and problem constants */
 
 typedef struct {
@@ -174,8 +179,8 @@ static int PrecSetupBD(N_Vector cc, N_Vector cscale,
                        N_Vector fval, N_Vector fscale,
                        void *user_data);
 
-static int PrecSolveBD(N_Vector cc, N_Vector cscale, 
-                       N_Vector fval, N_Vector fscale, 
+static int PrecSolveBD(N_Vector cc, N_Vector cscale,
+                       N_Vector fval, N_Vector fscale,
                        N_Vector vv, void *user_data);
 
 /* Private Helper Functions */
@@ -184,14 +189,14 @@ static UserData AllocUserData(void);
 static void InitUserData(UserData data);
 static void FreeUserData(UserData data);
 static void SetInitialProfiles(N_Vector cc, N_Vector sc);
-static void PrintHeader(int globalstrategy, int maxl, int maxlrst, 
+static void PrintHeader(int globalstrategy, int maxl, int maxlrst,
                         realtype fnormtol, realtype scsteptol);
 static void PrintOutput(N_Vector cc);
 static void PrintFinalStats(void *kmem);
-static void WebRate(realtype xx, realtype yy, realtype *cxy, realtype *ratesxy, 
+static void WebRate(realtype xx, realtype yy, realtype *cxy, realtype *ratesxy,
                     void *user_data);
 static realtype DotProd(sunindextype size, realtype *x1, realtype *x2);
-static int check_flag(void *flagvalue, const char *funcname, int opt);
+static int check_retval(void *retvalvalue, const char *funcname, int opt);
 
 /*
  *--------------------------------------------------------------------
@@ -201,11 +206,12 @@ static int check_flag(void *flagvalue, const char *funcname, int opt);
 
 int main(int argc, char *argv[])
 {
+  SUNContext sunctx;
   int globalstrategy;
   realtype fnormtol, scsteptol;
   N_Vector cc, sc, constraints;
   UserData data;
-  int flag, maxl, maxlrst;
+  int retval, maxl, maxlrst;
   void *kmem;
   SUNLinearSolver LS;
   int num_threads;
@@ -215,7 +221,11 @@ int main(int argc, char *argv[])
   LS = NULL;
   data = NULL;
 
-  /* Allocate memory, and set problem data, initial values, tolerances */ 
+  /* Create the SUNDIALS context that all SUNDIALS objects require */
+  retval = SUNContext_Create(NULL, &sunctx);
+  if (check_retval(&retval, "SUNContext_Create", 1)) return(1);
+
+  /* Allocate memory, and set problem data, initial values, tolerances */
   globalstrategy = KIN_NONE;
 
   /* Set the number of threads to use */
@@ -224,23 +234,23 @@ int main(int argc, char *argv[])
   num_threads = omp_get_max_threads();    /* Overwrite with OMP_NUM_THREADS environment variable */
 #endif
   if (argc > 1)        /* overwrithe with command line value, if supplied */
-    num_threads = strtol(argv[1], NULL, 0);
+    num_threads = (int) strtol(argv[1], NULL, 0);
 
   data = AllocUserData();
-  if (check_flag((void *)data, "AllocUserData", 2)) return(1);
+  if (check_retval((void *)data, "AllocUserData", 2)) return(1);
   InitUserData(data);
   data->nthreads = num_threads;
 
   /* Create serial vectors of length NEQ */
-  cc = N_VNew_OpenMP(NEQ, num_threads);
-  if (check_flag((void *)cc, "N_VNew_OpenMP", 0)) return(1);
-  sc = N_VNew_OpenMP(NEQ, num_threads);
-  if (check_flag((void *)sc, "N_VNew_OpenMP", 0)) return(1);
-  data->rates = N_VNew_OpenMP(NEQ, num_threads);
-  if (check_flag((void *)data->rates, "N_VNew_OpenMP", 0)) return(1);
+  cc = N_VNew_OpenMP(NEQ, num_threads, sunctx);
+  if (check_retval((void *)cc, "N_VNew_OpenMP", 0)) return(1);
+  sc = N_VNew_OpenMP(NEQ, num_threads, sunctx);
+  if (check_retval((void *)sc, "N_VNew_OpenMP", 0)) return(1);
+  data->rates = N_VNew_OpenMP(NEQ, num_threads, sunctx);
+  if (check_retval((void *)data->rates, "N_VNew_OpenMP", 0)) return(1);
 
-  constraints = N_VNew_OpenMP(NEQ, num_threads);
-  if (check_flag((void *)constraints, "N_VNew_OpenMP", 0)) return(1);
+  constraints = N_VNew_OpenMP(NEQ, num_threads, sunctx);
+  if (check_retval((void *)constraints, "N_VNew_OpenMP", 0)) return(1);
   N_VConst(TWO, constraints);
 
   SetInitialProfiles(cc, sc);
@@ -249,69 +259,70 @@ int main(int argc, char *argv[])
 
   /* Call KINCreate/KINInit to initialize KINSOL.
      A pointer to KINSOL problem memory is returned and stored in kmem. */
-  kmem = KINCreate();
-  if (check_flag((void *)kmem, "KINCreate", 0)) return(1);
+  kmem = KINCreate(sunctx);
+  if (check_retval((void *)kmem, "KINCreate", 0)) return(1);
 
   /* Vector cc passed as template vector. */
-  flag = KINInit(kmem, func, cc);
-  if (check_flag(&flag, "KINInit", 1)) return(1);
+  retval = KINInit(kmem, func, cc);
+  if (check_retval(&retval, "KINInit", 1)) return(1);
 
-  flag = KINSetUserData(kmem, data);
-  if (check_flag(&flag, "KINSetUserData", 1)) return(1);
-  flag = KINSetConstraints(kmem, constraints);
-  if (check_flag(&flag, "KINSetConstraints", 1)) return(1);
-  flag = KINSetFuncNormTol(kmem, fnormtol);
-  if (check_flag(&flag, "KINSetFuncNormTol", 1)) return(1);
-  flag = KINSetScaledStepTol(kmem, scsteptol);
-  if (check_flag(&flag, "KINSetScaledStepTol", 1)) return(1);
+  retval = KINSetUserData(kmem, data);
+  if (check_retval(&retval, "KINSetUserData", 1)) return(1);
+  retval = KINSetConstraints(kmem, constraints);
+  if (check_retval(&retval, "KINSetConstraints", 1)) return(1);
+  retval = KINSetFuncNormTol(kmem, fnormtol);
+  if (check_retval(&retval, "KINSetFuncNormTol", 1)) return(1);
+  retval = KINSetScaledStepTol(kmem, scsteptol);
+  if (check_retval(&retval, "KINSetScaledStepTol", 1)) return(1);
 
   /* We no longer need the constraints vector since KINSetConstraints
      creates a private copy for KINSOL to use. */
-  N_VDestroy_OpenMP(constraints);
+  N_VDestroy(constraints);
 
 
-  /* Create SUNLinSol_SPGMR object with right preconditioning and the 
+  /* Create SUNLinSol_SPGMR object with right preconditioning and the
      maximum Krylov dimension maxl */
-  maxl = 15; 
-  LS = SUNLinSol_SPGMR(cc, PREC_RIGHT, maxl);
-  if(check_flag((void *)LS, "SUNLinSol_SPGMR", 0)) return(1);
+  maxl = 15;
+  LS = SUNLinSol_SPGMR(cc, SUN_PREC_RIGHT, maxl, sunctx);
+  if(check_retval((void *)LS, "SUNLinSol_SPGMR", 0)) return(1);
 
   /* Attach the linear solver to KINSOL */
-  flag = KINSetLinearSolver(kmem, LS, NULL);
-  if (check_flag(&flag, "KINSetLinearSolver", 1)) return 1;
+  retval = KINSetLinearSolver(kmem, LS, NULL);
+  if (check_retval(&retval, "KINSetLinearSolver", 1)) return 1;
 
   /* Set the maximum number of restarts */
   maxlrst = 2;
-  flag = SUNLinSol_SPGMRSetMaxRestarts(LS, maxlrst);
-  if (check_flag(&flag, "SUNLinSol_SPGMRSetMaxRestarts", 1)) return(1);
+  retval = SUNLinSol_SPGMRSetMaxRestarts(LS, maxlrst);
+  if (check_retval(&retval, "SUNLinSol_SPGMRSetMaxRestarts", 1)) return(1);
 
   /* Specify the preconditioner setup and solve routines */
-  flag = KINSetPreconditioner(kmem, PrecSetupBD, PrecSolveBD);
-  if (check_flag(&flag, "KINSetPreconditioner", 1)) return(1);
+  retval = KINSetPreconditioner(kmem, PrecSetupBD, PrecSolveBD);
+  if (check_retval(&retval, "KINSetPreconditioner", 1)) return(1);
 
   /* Print out the problem size, solution parameters, initial guess. */
   PrintHeader(globalstrategy, maxl, maxlrst, fnormtol, scsteptol);
 
   /* Call KINSol and print output concentration profile */
-  flag = KINSol(kmem,           /* KINSol memory block */
+  retval = KINSol(kmem,           /* KINSol memory block */
                 cc,             /* initial guess on input; solution vector */
                 globalstrategy, /* global strategy choice */
                 sc,             /* scaling vector for the variable cc */
                 sc);            /* scaling vector for function values fval */
-  if (check_flag(&flag, "KINSol", 1)) return(1);
+  if (check_retval(&retval, "KINSol", 1)) return(1);
 
   printf("\n\nComputed equilibrium species concentrations:\n");
   PrintOutput(cc);
 
-  /* Print final statistics and free memory */  
+  /* Print final statistics and free memory */
   PrintFinalStats(kmem);
   printf("num_threads = %i\n", num_threads);
 
-  N_VDestroy_OpenMP(cc);
-  N_VDestroy_OpenMP(sc);
+  N_VDestroy(cc);
+  N_VDestroy(sc);
   KINFree(&kmem);
   SUNLinSolFree(LS);
   FreeUserData(data);
+  SUNContext_Free(&sunctx);
 
   return(0);
 }
@@ -329,8 +340,8 @@ int main(int argc, char *argv[])
  *--------------------------------------------------------------------
  */
 
-/* 
- * System function for predator-prey system 
+/*
+ * System function for predator-prey system
  */
 
 static int func(N_Vector cc, N_Vector fval, void *user_data)
@@ -338,20 +349,20 @@ static int func(N_Vector cc, N_Vector fval, void *user_data)
   realtype xx, yy, delx, dely, *cxy, *rxy, *fxy, dcyli, dcyui, dcxli, dcxri;
   sunindextype jx, jy, is, idyu, idyl, idxr, idxl;
   UserData data;
-  
+
   data = (UserData)user_data;
   delx = data->dx;
   dely = data->dy;
-  
+
   /* Loop over all mesh points, evaluating rate array at each point*/
   for (jy = 0; jy < MY; jy++) {
-    
+
     yy = dely*jy;
 
     /* Set lower/upper index shifts, special at boundaries. */
     idyl = (jy != 0   ) ? NSMX : -NSMX;
     idyu = (jy != MY-1) ? NSMX : -NSMX;
-    
+
     for (jx = 0; jx < MX; jx++) {
 
       xx = delx*jx;
@@ -368,30 +379,30 @@ static int func(N_Vector cc, N_Vector fval, void *user_data)
       WebRate(xx, yy, cxy, rxy, user_data);
 
       for(is = 0; is < NUM_SPECIES; is++) {
-        
+
         /* Differencing in x direction */
         dcyli = *(cxy+is) - *(cxy - idyl + is) ;
         dcyui = *(cxy + idyu + is) - *(cxy+is);
-        
+
         /* Differencing in y direction */
         dcxli = *(cxy+is) - *(cxy - idxl + is);
         dcxri = *(cxy + idxr +is) - *(cxy+is);
-        
+
         /* Compute the total rate value at (xx,yy) */
         fxy[is] = (coy)[is] * (dcyui - dcyli) +
           (cox)[is] * (dcxri - dcxli) + rxy[is];
-        
+
       } /* end of is loop */
-      
+
     } /* end of jx loop */
-    
+
   } /* end of jy loop */
 
   return(0);
 }
 
 /*
- * Preconditioner setup routine. Generate and preprocess P. 
+ * Preconditioner setup routine. Generate and preprocess P.
  */
 
 static int PrecSetupBD(N_Vector cc, N_Vector cscale,
@@ -402,21 +413,23 @@ static int PrecSetupBD(N_Vector cc, N_Vector cscale,
   realtype *cxy, *scxy, **Pxy, *ratesxy, *Pxycol, perturb_rates[NUM_SPECIES];
   sunindextype i, j, jx, jy, ret;
   UserData data;
-  
+
+  i = j = jx = jy = ret = 0;
+
   data = (UserData) user_data;
   delx = data->dx;
   dely = data->dy;
-  
+
   uround = data->uround;
   sqruround = data->sqruround;
   fac = N_VWL2Norm(fval, fscale);
   r0 = THOUSAND * uround * fac * NEQ;
   if(r0 == ZERO) r0 = ONE;
-  
+
   /* Loop over spatial points; get size NUM_SPECIES Jacobian block at each */
   for (jy = 0; jy < MY; jy++) {
     yy = jy*dely;
-    
+
     for (jx = 0; jx < MX; jx++) {
       xx = jx*delx;
       Pxy = (data->P)[jx][jy];
@@ -426,56 +439,58 @@ static int PrecSetupBD(N_Vector cc, N_Vector cscale,
 
       /* Compute difference quotients of interaction rate fn. */
       for (j = 0; j < NUM_SPECIES; j++) {
-        
+
         csave = cxy[j];  /* Save the j,jx,jy element of cc */
-        r = SUNMAX(sqruround*SUNRabs(csave), r0/scxy[j]);
+        r = MAX(sqruround*fabs(csave), r0/scxy[j]);
         cxy[j] += r; /* Perturb the j,jx,jy element of cc */
         fac = ONE/r;
-        
+
         WebRate(xx, yy, cxy, perturb_rates, data);
-        
+
         /* Restore j,jx,jy element of cc */
         cxy[j] = csave;
-        
+
         /* Load the j-th column of difference quotients */
         Pxycol = Pxy[j];
-	
+
 #pragma omp parallel for default(shared) private(i)
         for (i = 0; i < NUM_SPECIES; i++)
           Pxycol[i] = (perturb_rates[i] - ratesxy[i]) * fac;
-        
+
       } /* end of j loop */
-      
+
       /* Do LU decomposition of size NUM_SPECIES preconditioner block */
-      ret = denseGETRF(Pxy, NUM_SPECIES, NUM_SPECIES, (data->pivot)[jx][jy]);
+      ret = SUNDlsMat_denseGETRF(Pxy, NUM_SPECIES, NUM_SPECIES, (data->pivot)[jx][jy]);
       if (ret != 0) return(1);
-      
+
     } /* end of jx loop */
-    
+
   } /* end of jy loop */
-  
+
   return(0);
 }
 
 /*
- * Preconditioner solve routine 
+ * Preconditioner solve routine
  */
 
-static int PrecSolveBD(N_Vector cc, N_Vector cscale, 
-                       N_Vector fval, N_Vector fscale, 
+static int PrecSolveBD(N_Vector cc, N_Vector cscale,
+                       N_Vector fval, N_Vector fscale,
                        N_Vector vv, void *user_data)
 {
   realtype **Pxy, *vxy;
   sunindextype *piv, jx, jy;
   UserData data;
-  
+
+  jx = jy = 0;
+
   data = (UserData)user_data;
-  
-#pragma omp parallel for collapse(2) default(shared) private(jx, jy, Pxy, piv, vxy) schedule(static) 
+
+#pragma omp parallel for collapse(2) default(shared) private(jx, jy, Pxy, piv, vxy) schedule(static)
   for (jx=0; jx<MX; jx++) {
-    
+
     for (jy=0; jy<MY; jy++) {
-      
+
       /* For each (jx,jy), solve a linear system of size NUM_SPECIES.
          vxy is the address of the corresponding portion of the vector vv;
          Pxy is the address of the corresponding block of the matrix P;
@@ -483,47 +498,47 @@ static int PrecSolveBD(N_Vector cc, N_Vector cscale,
       vxy = IJ_Vptr(vv,jx,jy);
       Pxy = (data->P)[jx][jy];
       piv = (data->pivot)[jx][jy];
-      denseGETRS(Pxy, NUM_SPECIES, piv, vxy);
-      
+      SUNDlsMat_denseGETRS(Pxy, NUM_SPECIES, piv, vxy);
+
     } /* end of jy loop */
-    
+
   } /* end of jx loop */
-  
+
   return(0);
 }
 
 /*
- * Interaction rate function routine 
+ * Interaction rate function routine
  */
 
-static void WebRate(realtype xx, realtype yy, realtype *cxy, realtype *ratesxy, 
+static void WebRate(realtype xx, realtype yy, realtype *cxy, realtype *ratesxy,
                     void *user_data)
 {
   sunindextype i;
   realtype fac;
   UserData data;
-  
+
   data = (UserData)user_data;
-  
+
   for (i = 0; i<NUM_SPECIES; i++)
     ratesxy[i] = DotProd(NUM_SPECIES, cxy, acoef[i]);
-  
+
   fac = ONE + ALPHA * xx * yy;
-  
-#pragma omp parallel for default(shared) private(i) 
+
+#pragma omp parallel for default(shared) private(i)
   for (i = 0; i < NUM_SPECIES; i++)
     ratesxy[i] = cxy[i] * ( bcoef[i] * fac + ratesxy[i] );
 }
 
 /*
- * Dot product routine for realtype arrays 
+ * Dot product routine for realtype arrays
  */
 
 static realtype DotProd(sunindextype size, realtype *x1, realtype *x2)
 {
   sunindextype i;
   realtype *xx1, *xx2, temp = ZERO;
-  
+
   xx1 = x1; xx2 = x2;
   for (i = 0; i < size; i++) temp += (*xx1++) * (*xx2++);
 
@@ -537,7 +552,7 @@ static realtype DotProd(sunindextype size, realtype *x1, realtype *x2)
  */
 
 /*
- * Allocate memory for data structure of type UserData 
+ * Allocate memory for data structure of type UserData
  */
 
 static UserData AllocUserData(void)
@@ -546,24 +561,24 @@ static UserData AllocUserData(void)
   UserData data;
 
   data = (UserData) malloc(sizeof *data);
-  
+
   for (jx=0; jx < MX; jx++) {
     for (jy=0; jy < MY; jy++) {
-      (data->P)[jx][jy] = newDenseMat(NUM_SPECIES, NUM_SPECIES);
-      (data->pivot)[jx][jy] = newIndexArray(NUM_SPECIES);
+      (data->P)[jx][jy] = SUNDlsMat_newDenseMat(NUM_SPECIES, NUM_SPECIES);
+      (data->pivot)[jx][jy] = SUNDlsMat_newIndexArray(NUM_SPECIES);
     }
   }
 
-  acoef = newDenseMat(NUM_SPECIES, NUM_SPECIES);
+  acoef = SUNDlsMat_newDenseMat(NUM_SPECIES, NUM_SPECIES);
   bcoef = (realtype *)malloc(NUM_SPECIES * sizeof(realtype));
   cox   = (realtype *)malloc(NUM_SPECIES * sizeof(realtype));
   coy   = (realtype *)malloc(NUM_SPECIES * sizeof(realtype));
-  
+
   return(data);
 }
 
 /*
- * Load problem constants in data 
+ * Load problem constants in data
  */
 
 static void InitUserData(UserData data)
@@ -580,7 +595,7 @@ static void InitUserData(UserData data)
   data->dx = (data->ax)/(MX-1);
   data->dy = (data->ay)/(MY-1);
   data->uround = UNIT_ROUNDOFF;
-  data->sqruround = SUNRsqrt(data->uround);
+  data->sqruround = sqrt(data->uround);
 
   /* Set up the coefficients a and b plus others found in the equations */
   np = data->np;
@@ -617,7 +632,7 @@ static void InitUserData(UserData data)
 }
 
 /*
- * Free data memory 
+ * Free data memory
  */
 
 static void FreeUserData(UserData data)
@@ -626,21 +641,21 @@ static void FreeUserData(UserData data)
 
   for (jx=0; jx < MX; jx++) {
     for (jy=0; jy < MY; jy++) {
-      destroyMat((data->P)[jx][jy]);
-      destroyArray((data->pivot)[jx][jy]);
+      SUNDlsMat_destroyMat((data->P)[jx][jy]);
+      SUNDlsMat_destroyArray((data->pivot)[jx][jy]);
     }
   }
 
-  destroyMat(acoef);
+  SUNDlsMat_destroyMat(acoef);
   free(bcoef);
   free(cox);
   free(coy);
-  N_VDestroy_OpenMP(data->rates);
+  N_VDestroy(data->rates);
   free(data);
 }
 
 /*
- * Set initial conditions in cc 
+ * Set initial conditions in cc
  */
 
 static void SetInitialProfiles(N_Vector cc, N_Vector sc)
@@ -648,7 +663,7 @@ static void SetInitialProfiles(N_Vector cc, N_Vector sc)
   int i, jx, jy;
   realtype *cloc, *sloc;
   realtype  ctemp[NUM_SPECIES], stemp[NUM_SPECIES];
-  
+
   /* Initialize arrays ctemp and stemp used in the loading process */
   for (i = 0; i < NUM_SPECIES/2; i++) {
     ctemp[i] = PREYIN;
@@ -672,11 +687,11 @@ static void SetInitialProfiles(N_Vector cc, N_Vector sc)
   }
 }
 
-/* 
+/*
  * Print first lines of output (problem description)
  */
 
-static void PrintHeader(int globalstrategy, int maxl, int maxlrst, 
+static void PrintHeader(int globalstrategy, int maxl, int maxlrst,
                         realtype fnormtol, realtype scsteptol)
 {
   printf("\nPredator-prey test problem --  KINSol (OpenMP version)\n\n");
@@ -689,7 +704,7 @@ static void PrintHeader(int globalstrategy, int maxl, int maxlrst,
          maxl, maxlrst);
   printf("Preconditioning uses interaction-only block-diagonal matrix\n");
   printf("Positivity constraints imposed on all components \n");
-#if defined(SUNDIALS_EXTENDED_PRECISION) 
+#if defined(SUNDIALS_EXTENDED_PRECISION)
   printf("Tolerance parameters:  fnormtol = %Lg   scsteptol = %Lg\n",
          fnormtol, scsteptol);
 #elif defined(SUNDIALS_DOUBLE_PRECISION)
@@ -701,30 +716,30 @@ static void PrintHeader(int globalstrategy, int maxl, int maxlrst,
 #endif
 
   printf("\nInitial profile of concentration\n");
-#if defined(SUNDIALS_EXTENDED_PRECISION) 
-  printf("At all mesh points:  %Lg %Lg %Lg   %Lg %Lg %Lg\n", 
+#if defined(SUNDIALS_EXTENDED_PRECISION)
+  printf("At all mesh points:  %Lg %Lg %Lg   %Lg %Lg %Lg\n",
          PREYIN, PREYIN, PREYIN,
          PREDIN, PREDIN, PREDIN);
 #elif defined(SUNDIALS_DOUBLE_PRECISION)
-  printf("At all mesh points:  %g %g %g   %g %g %g\n", 
+  printf("At all mesh points:  %g %g %g   %g %g %g\n",
          PREYIN, PREYIN, PREYIN,
          PREDIN, PREDIN, PREDIN);
 #else
-  printf("At all mesh points:  %g %g %g   %g %g %g\n", 
+  printf("At all mesh points:  %g %g %g   %g %g %g\n",
          PREYIN, PREYIN, PREYIN,
          PREDIN, PREDIN, PREDIN);
 #endif
 }
 
 /*
- * Print sampled values of current cc 
+ * Print sampled values of current cc
  */
 
 static void PrintOutput(N_Vector cc)
 {
   int is, jx, jy;
   realtype *ct;
-  
+
   jy = 0; jx = 0;
   ct = IJ_Vptr(cc,jx,jy);
   printf("\nAt bottom left:");
@@ -740,7 +755,7 @@ static void PrintOutput(N_Vector cc)
     printf(" %g",ct[is]);
 #endif
   }
-  
+
   jy = MY-1; jx = MX-1;
   ct = IJ_Vptr(cc,jx,jy);
   printf("\n\nAt top right:");
@@ -760,28 +775,28 @@ static void PrintOutput(N_Vector cc)
 }
 
 /*
- * Print final statistics contained in iopt 
+ * Print final statistics contained in iopt
  */
 
 static void PrintFinalStats(void *kmem)
 {
   long int nni, nfe, nli, npe, nps, ncfl, nfeSG;
-  int flag;
+  int retval;
 
-  flag = KINGetNumNonlinSolvIters(kmem, &nni);
-  check_flag(&flag, "KINGetNumNonlinSolvIters", 1);
-  flag = KINGetNumFuncEvals(kmem, &nfe);
-  check_flag(&flag, "KINGetNumFuncEvals", 1);
-  flag = KINGetNumLinIters(kmem, &nli);
-  check_flag(&flag, "KINGetNumLinIters", 1);
-  flag = KINGetNumPrecEvals(kmem, &npe);
-  check_flag(&flag, "KINGetNumPrecEvals", 1);
-  flag = KINGetNumPrecSolves(kmem, &nps);
-  check_flag(&flag, "KINGetNumPrecSolves", 1);
-  flag = KINGetNumLinConvFails(kmem, &ncfl);
-  check_flag(&flag, "KINGetNumLinConvFails", 1);
-  flag = KINGetNumLinFuncEvals(kmem, &nfeSG);
-  check_flag(&flag, "KINGetNumLinFuncEvals", 1);
+  retval = KINGetNumNonlinSolvIters(kmem, &nni);
+  check_retval(&retval, "KINGetNumNonlinSolvIters", 1);
+  retval = KINGetNumFuncEvals(kmem, &nfe);
+  check_retval(&retval, "KINGetNumFuncEvals", 1);
+  retval = KINGetNumLinIters(kmem, &nli);
+  check_retval(&retval, "KINGetNumLinIters", 1);
+  retval = KINGetNumPrecEvals(kmem, &npe);
+  check_retval(&retval, "KINGetNumPrecEvals", 1);
+  retval = KINGetNumPrecSolves(kmem, &nps);
+  check_retval(&retval, "KINGetNumPrecSolves", 1);
+  retval = KINGetNumLinConvFails(kmem, &ncfl);
+  check_retval(&retval, "KINGetNumLinConvFails", 1);
+  retval = KINGetNumLinFuncEvals(kmem, &nfeSG);
+  check_retval(&retval, "KINGetNumLinFuncEvals", 1);
 
   printf("Final Statistics.. \n");
   printf("nni    = %5ld    nli   = %5ld\n", nni, nli);
@@ -794,37 +809,37 @@ static void PrintFinalStats(void *kmem)
  * Check function return value...
  *    opt == 0 means SUNDIALS function allocates memory so check if
  *             returned NULL pointer
- *    opt == 1 means SUNDIALS function returns a flag so check if
- *             flag >= 0
+ *    opt == 1 means SUNDIALS function returns a retval so check if
+ *             retval >= 0
  *    opt == 2 means function allocates memory so check if returned
- *             NULL pointer 
+ *             NULL pointer
  */
 
-static int check_flag(void *flagvalue, const char *funcname, int opt)
+static int check_retval(void *retvalvalue, const char *funcname, int opt)
 {
-  int *errflag;
+  int *errretval;
 
   /* Check if SUNDIALS function returned NULL pointer - no memory allocated */
-  if (opt == 0 && flagvalue == NULL) {
-    fprintf(stderr, 
+  if (opt == 0 && retvalvalue == NULL) {
+    fprintf(stderr,
             "\nSUNDIALS_ERROR: %s() failed - returned NULL pointer\n\n",
 	    funcname);
     return(1);
   }
 
-  /* Check if flag < 0 */
+  /* Check if retval < 0 */
   else if (opt == 1) {
-    errflag = (int *) flagvalue;
-    if (*errflag < 0) {
+    errretval = (int *) retvalvalue;
+    if (*errretval < 0) {
       fprintf(stderr,
-              "\nSUNDIALS_ERROR: %s() failed with flag = %d\n\n",
-	      funcname, *errflag);
+              "\nSUNDIALS_ERROR: %s() failed with retval = %d\n\n",
+	      funcname, *errretval);
       return(1);
     }
   }
 
   /* Check if function returned NULL pointer - no memory allocated */
-  else if (opt == 2 && flagvalue == NULL) {
+  else if (opt == 2 && retvalvalue == NULL) {
     fprintf(stderr,
             "\nMEMORY_ERROR: %s() failed - returned NULL pointer\n\n",
 	    funcname);
